## Simulation & Tests

NINA's test suite is composed of three components:

1. **CPU Pipeline:** basic testbench to verify the pipeline
2. **RISC-V ISA:** set of tests for validating compliance with RISC-V. Provided by RISC-V International
3. **C Programs:** set of C programs to verify functionality and benchmarking

It is necessary to install **Icarus Verilog** (`iverilog`) and the **GCC RISC-V Toolchain** (`riscv64-unknown-elf-*`).
 
#### CPU Pipeline Tests

This testbench tests all the RV32I instructions, plus does basic pipeline hazards tests. To run the testbench: 

```bash
make -B sim/cpu_tb.fst
cat sim/cpu_tb.log       # Print the results
```

### RISC-V ISA Tests

This test suite verifies compliance with all the instructions in RV32I. To compile the entire suite:

```shell
cd software/riscv-isa-tests && make
```

Then, run the test suite:
```shell
cd hardware
make isa-tests                       # to run all ISA tests
make sim/isa/lw.fst                  # to run a specific test
grep -r -i "failed" sim/isa/*.log    # to check for failures
grep -r -i "passed" sim/isa/*.log    # to check for passing tests
grep -r -i "timeout" sim/isa/*.log   # to check for tests that timed out
```

### C Programs

The C programs and related testbench are located in `software/c_tests`. If you modify the files, you should build them by running `make`.
The available tests include: `strcmp`  `vecadd`, `fib`, `sum`, `replace`, `cachetest`, `mmult` and `benchmark`. The latter is used the program used to the processor.

To run the tests:
```shell
cd hardware
make c-tests                         # run all C tests
make sim/c_tests/sum.fst             # run the sum C test
```

### Running All Tests

We provide a script to run all tests at once:

To run all tests 

```bash
python3 run_all_sims.py --simulator iverilog
```

Note that you should build any tests before using this script.

## Cleaning Simulation Outputs

To clean simulation outputs: `make clean-sim`
