module load_extender (
  input [31:0] data_in,
  input [2:0] funct3,
  input [1:0] addr,

  output reg [31:0] data_out
);

  wire bit funct3_unsigned_bit;
  assign funct3_unsigned_bit = funct3[2];

  wire [1:0] load_width;
  assign load_width = funct3[1:0];

  wire [1:0] position;
  assign position = addr[1:0];

  always @* begin
    // TODO: Verify is clearing the output is indeed needed
    data_out = 32'h0; 
    
    if (funct3_unsigned_bit) begin
      case (load_width)
        // Load byte
        2'b00: begin
          case (position)
            2'b00: data_out = 32'h000000ff & data_in[7:0];
            2'b01: data_out = 32'h000000ff & data_in[15:8];
            2'b10: data_out = 32'h000000ff & data_in[23:16];
            2'b11: data_out = 32'h000000ff & data_in[31:24];
          endcase
        end
        // Load half-word
        2'b01: begin
          case (position)
            2'b00: data_out = 32'h0000ffff & data_in[15:0];
            2'b10: data_out = 32'h0000ffff & data_in[31:16];
          endcase
        end
      endcase
    end
    else begin
      case (load_width)
        // Load byte
        2'b00: begin
          case (position)
            2'b00: data_out = {{24{data_in[7]}}, data_in[7:0]};
            2'b01: data_out = {{24{data_in[15]}}, data_in[15:8]};
            2'b10: data_out = {{24{data_in[23]}}, data_in[23:16]};
            2'b11: data_out = {{24{data_in[31]}}, data_in[31:24]};
          endcase
        end
        // Load half-word
        2'b01: begin
          case (position)
            2'b00: data_out = {{16{data_in[15]}}, data_in[15:0]};
            2'b10: data_out = {{16{data_in[31]}}, data_in[31:16]};
          endcase
        end
        // Load word
        2'b10: data_out = data_in;
      endcase
    end
  end

endmodule
